"""
Authors: Peter Mawhorter
Consulted:
Date: 2022-10-24
Purpose: Testing file for the trainInventory task.
"""

from optimism import (
    freshTestSuite,
    testFunctionMaybe,
    testBlock,
    Loop,
    showOverview
)

import trainInventory

# Trains to test with

twoCars = [
    [('grain', 100000)],
    [('chairs', 2000), ('sofas', 2300), ('chairs', 1000), ('tables', 4000)]
]

grainTrain = [
    [('wheat', 122000)],
    [('wheat', 148000)],
    [('semolina', 90000)],
    [('semolina', 93000)],
    [('semolina', 89000)],
    [('spelt', 91000), ('barley', 24000)],
]

miscTrain = [
    [('electronics', 2000), ('cloth', 40000), ('toys', 8000)],
    [('racoons', 8), ('opossum', 5), ('rabbits', 37), ('deer', 82)],
    [('steel', 22000), ('iron', 45000), ('cobalt', 12000)],
    [('helium', 4000), ('electronics', 50)],
]

# Tests

freshTestSuite('printInventory')
tester = testFunctionMaybe(trainInventory, 'printInventory')
tester.checkCodeContains(Loop().contains(Loop()))
tester.case(twoCars).checkPrintedLines(
    'Car #0',
    '  100000 kg grain',
    'Car #1',
    '  2000 kg chairs',
    '  2300 kg sofas',
    '  1000 kg chairs',
    '  4000 kg tables',
)
tester.case(grainTrain).checkPrintedLines(
    'Car #0',
    '  122000 kg wheat',
    'Car #1',
    '  148000 kg wheat',
    'Car #2',
    '  90000 kg semolina',
    'Car #3',
    '  93000 kg semolina',
    'Car #4',
    '  89000 kg semolina',
    'Car #5',
    '  91000 kg spelt',
    '  24000 kg barley',
)
tester.case(miscTrain).checkPrintedLines(
    'Car #0',
    '  2000 kg electronics',
    '  40000 kg cloth',
    '  8000 kg toys',
    'Car #1',
    '  8 kg racoons',
    '  5 kg opossum',
    '  37 kg rabbits',
    '  82 kg deer',
    'Car #2',
    '  22000 kg steel',
    '  45000 kg iron',
    '  12000 kg cobalt',
    'Car #3',
    '  4000 kg helium',
    '  50 kg electronics',
)

freshTestSuite('totalWeightOf')
tester = testFunctionMaybe(trainInventory, 'totalWeightOf')
tester.checkCodeContains(Loop().contains(Loop()))
tester.case(twoCars, 'grain').checkReturnValue(100000)
tester.case(twoCars, 'chairs').checkReturnValue(3000)
tester.case(twoCars, 'sofas').checkReturnValue(2300)
tester.case(twoCars, 'mice').checkReturnValue(0)
tester.case(grainTrain, 'wheat').checkReturnValue(270000)
tester.case(miscTrain, 'electronics').checkReturnValue(2050)

freshTestSuite('listItems')
tester = testFunctionMaybe(trainInventory, 'listItems')
tester.checkCodeContains(Loop().contains(Loop()))
tester.case(twoCars, 'grain').checkReturnValue([('grain', 100000)])
tester.case(grainTrain, 'semolina').checkReturnValue(
    [
        ('semolina', 90000),
        ('semolina', 93000),
        ('semolina', 89000),
    ]
)
tester.case(miscTrain, 'electronics').checkReturnValue(
    [
        ('electronics', 2000),
        ('electronics', 50),
    ]
)

freshTestSuite('listWeights')
tester = testFunctionMaybe(trainInventory, 'listWeights')
tester.checkCodeContains(Loop().contains(Loop()))
tester.case(twoCars).checkReturnValue([100000, 9300])
tester.case(grainTrain).checkReturnValue(
    [
        122000,
        148000,
        90000,
        93000,
        89000,
        115000,
    ]
)
tester.case(miscTrain).checkReturnValue(
    [
        50000,
        132,
        79000,
        4050,
    ]
)

freshTestSuite('heaviestCar')
tester = testFunctionMaybe(trainInventory, 'heaviestCar')
tester.case(twoCars).checkReturnValue(0)
tester.case(grainTrain).checkReturnValue(1)
tester.case(miscTrain).checkReturnValue(2)

# Note: These tests are not applicable to the current version of the
# problem, so they're commented out.
## freshTestSuite('itemWeight')
## tester = testFunctionMaybe(trainInventory, 'itemWeight')
## tester.case(('grain', 100000)).checkReturnValue(100000)
## tester.case(('electronics', 2000)).checkReturnValue(2000)
## tester.case(("doesn't matter", 0)).checkReturnValue(0)
## 
## freshTestSuite('inventoryByWeight')
## tester = testFunctionMaybe(trainInventory, 'inventoryByWeight')
## tester.checkCodeContains(Loop().contains(Loop()))
## tester.case(twoCars).checkReturnValue(
##     [
##         ('chairs', 1000),
##         ('chairs', 2000),
##         ('sofas', 2300),
##         ('tables', 4000),
##         ('grain', 100000)
##     ]
## )
## tester.case(grainTrain).checkReturnValue(
##     [
##         ('barley', 24000),
##         ('semolina', 89000),
##         ('semolina', 90000),
##         ('spelt', 91000),
##         ('semolina', 93000),
##         ('wheat', 122000),
##         ('wheat', 148000),
##     ]
## )
## tester.case(miscTrain).checkReturnValue(
##     [
##         ('opossum', 5),
##         ('racoons', 8),
##         ('rabbits', 37),
##         ('electronics', 50),
##         ('deer', 82),
##         ('electronics', 2000),
##         ('helium', 4000),
##         ('toys', 8000),
##         ('cobalt', 12000),
##         ('steel', 22000),
##         ('cloth', 40000),
##         ('iron', 45000),
##     ]
## )

freshTestSuite('repackCars')
tester = testFunctionMaybe(trainInventory, 'repackCars')
tester.checkCodeContains(Loop().contains(Loop()))
tester.case(twoCars, 5000).checkReturnValue(
    [
        [('grain', 100000)],
        [
            ('chairs', 2000),
            ('sofas', 2300),
        ],
        [
            ('chairs', 1000),
            ('tables', 4000)
        ]
    ]
)
tester.case(grainTrain, 200000).checkReturnValue(
    [
        [('wheat', 122000)],
        [('wheat', 148000)],
        [
            ('semolina', 90000),
            ('semolina', 93000),
        ],
        [
            ('semolina', 89000),
            ('spelt', 91000),
        ],
        [
            ('barley', 24000),
        ]
    ]
)
tester.case(grainTrain, 150000).checkReturnValue(
    [
        [('wheat', 122000)],
        [('wheat', 148000)],
        [('semolina', 90000)],
        [('semolina', 93000)],
        [('semolina', 89000)],
        [('spelt', 91000), ('barley', 24000)]
    ]
)
tester.case(grainTrain, 100000).checkReturnValue(
    [
        [('wheat', 122000)],
        [('wheat', 148000)],
        [('semolina', 90000)],
        [('semolina', 93000)],
        [('semolina', 89000)],
        [('spelt', 91000)],
        [('barley', 24000)]
    ]
)
tester.case(miscTrain, 3000).checkReturnValue(
    [
        [('electronics', 2000)],
        [('cloth', 40000)],
        [('toys', 8000)],
        [
            ('racoons', 8),
            ('opossum', 5),
            ('rabbits', 37),
            ('deer', 82),
        ],
        [('steel', 22000)],
        [('iron', 45000)],
        [('cobalt', 12000)],
        [('helium', 4000)],
        [('electronics', 50)],
    ]
)
tester.case(miscTrain, 10000).checkReturnValue(
    [
        [('electronics', 2000)],
        [('cloth', 40000)],
        [
            ('toys', 8000),
            ('racoons', 8),
            ('opossum', 5),
            ('rabbits', 37),
            ('deer', 82)
        ],
        [('steel', 22000)],
        [('iron', 45000)],
        [('cobalt', 12000)],
        [('helium', 4000), ('electronics', 50)],
    ]
)


# Tests for removeItems only if you defined it
if hasattr(trainInventory, 'removeItems'):
    import copy

    freshTestSuite('removeItems')
    tester = testBlock('result = removeItems(train, item)')
    case1 = tester.case(
        removeItems=trainInventory.removeItems,
        train=copy.deepcopy(twoCars),
        item='sofas'
    )
    case1.checkVariableValue('result', [('sofas', 2300)])
    case1.checkVariableValue(
        'train',
        [
            [('grain', 100000)],
            [('chairs', 2000), ('chairs', 1000), ('tables', 4000)]
        ]
    )
    case2 = tester.case(
        removeItems=trainInventory.removeItems,
        train=copy.deepcopy(twoCars),
        item='chairs'
    )
    case2.checkVariableValue('result', [('chairs', 2000), ('chairs', 1000)])
    case2.checkVariableValue(
        'train',
        [
            [('grain', 100000)],
            [('sofas', 2300), ('tables', 4000)]
        ]
    )
    case3 = tester.case(
        removeItems=trainInventory.removeItems,
        train=copy.deepcopy(grainTrain),
        item='semolina'
    )
    case3.checkVariableValue(
        'result',
        [('semolina', 90000), ('semolina', 93000), ('semolina', 89000)]
    )
    case3.checkVariableValue(
        'train',
        [
            [('wheat', 122000)],
            [('wheat', 148000)],
            [],
            [],
            [],
            [('spelt', 91000), ('barley', 24000)],
        ]
    )
    case4 = tester.case(
        removeItems=trainInventory.removeItems,
        train=copy.deepcopy(miscTrain),
        item='electronics'
    )
    case4.checkVariableValue(
        'result',
        [('electronics', 2000), ('electronics', 50)]
    )
    case4.checkVariableValue(
        'train',
        [
            [('cloth', 40000), ('toys', 8000)],
            [('racoons', 8), ('opossum', 5), ('rabbits', 37), ('deer', 82)],
            [('steel', 22000), ('iron', 45000), ('cobalt', 12000)],
            [('helium', 4000)],
        ]
    )

showOverview()
